<?php

/* ************************************************* *\
|* ================================================= *|
|* =       Copyright (c) 2012 Jacob Pollack.       = *|
|* ================================================= *|
|* = THIS SOFTWARE IS PRIVATE AND CANNOT BE FOUND  = *|
|* = ON THE PUBLIC DOMAIN. ANY DISTRIBUTION OR USE = *|
|* = OF THIS SOFTWARE WITHOUT THE EXPLICIT         = *|
|* = PERMISSION OF THE AUTHOR WILL RESULT IN LEGAL = *|
|* = ACTION BEING TAKEN.                           = *|
|* =                                               = *|
|* = THIS SOFTWARE IS CREATED FOR EDUCATIONAL      = *|
|* = PURPOSES ONLY. CHANGES OF ANY SORT MUST BE    = *|
|* = SUBMITTED TO THE RESPOSITORY ACCORDINGLY.     = *|
|* ================================================= *|
|* = @author    Jacob Pollack                      = *|
|* = @email     jacobpollack@jamach.com            = *|
|* = @date      05/08/2012 (DD/MM/YYYY)            = *|
|* =                                               = *|
|* = @contributors  none                           = *|
|* ================================================= *|
\*************************************************** */

/**
 * The cache class handles the caching of 'static', yet
 * dynamic content for the entire system.
 * 
 * @name    Cache Handler
 * @author  Jacob Pollack
 */

/**
 * @todo
 * - Implement a cache duration feature.
 * - Protect all files in the 'cache' directory.
 * - Possibly have encoded data serialized in an array so that
 * returning the data can be easier and with more flexibility
 * for logging.
 * - Implement support for memory caching (APC, MemCache, etc.).
 * - Better encoding/decoding algorithm (more than just serialize but something
 * fast).
 * - If the caching path does not exist, create it (constructor).
 * - Support caching for 'x' amount of time.
 * - Clear files in the cache directory if unused for 'x' amount of time.
 * - Better encryption for the file names (faster).
 */
final class cachehandler
{
    /**
     * Path to the cached directory.
     * 
     * @var string
     */
    private $path = null;
    
    /**
     * The extension of all of the cached files.
     * 
     * @var string 
     */
    private $extension = '.php';
    
    /**
     * Builds the file path for the cache directory.
     * 
     * @return void
     */
    final public function __construct($path)
    {
        $this->path = CWD . INCLUDES . $path;
    }
    
    /**
     * Checks if a cached entry exists by name (key).
     * 
     * @param   string  $name   The name of the cached entry (key).
     * 
     * @return  boolean
     */
    final public function file_exists($name)
    {
        if (file_exists($this->full_path($name)) == true)
        {
            return is_writable($this->full_path($name));
        }
        
        return false;
    }
    
    /**
     * Stores an object or array in the cache by name (key).
     * 
     * @param   string          $name   The name of the cached entry (key).
     * @param   object/array    $data   The data to be cached.
     * 
     * @return  boolean
     */
    final public function file_store($name, $data)
    {
        if ($this->file_exists($name) == false)
        {
            if ((is_object($data) == true) || (is_array($data) == true))
            {
                file_put_contents($this->full_path($name), $this->encode($data), LOCK_EX);
            
                return true;
            }
            else
            {
                trigger_error('Data being cached must be an object.');
            }
        }
        
        return false;
    }
    
    /**
     * Fetches a cached entry by name (key).
     * 
     * @param   string  $name   The name of the cached entry (key).
     * 
     * @return  any
     */
    final public function file_fetch($name)
    {
        return $this->decode(file_get_contents($this->full_path($name)));
    }
    
    /**
     * Encodes a data value by serializing it.
     * 
     * @param   any     $data   The data to be encoded.
     * 
     * @return  string
     */
    final private function encode($data)
    {
        return serialize($data);
    }
    
    /**
     * Decodes an encoded 'any' by unserializing it.
     * 
     * @param   string  $data   The data that is encoded.
     * 
     * @return  any
     */
    final private function decode($data)
    {
        return unserialize($data);
    }
    
    /**
     * Encrypts the cache name (key).
     * 
     * @param   string  $name   The name of the cached entry (key).
     * 
     * @return  string
     */
    final private function encrypt($name)
    {
        return md5($name);
    }
    
    /**
     * Grabs the full path to the cache directory and cache
     * entry.
     * 
     * @param   string  $name   The name of the cached entry (key).
     * 
     * @return  string
     */
    final private function full_path($name)
    {
        return $this->path . $this->encrypt($name) . $this->extension;
    }
    
    /**
     * Empty destructor.
     * 
     * @return  void
     */
    final public function __destruct() { }
}

?>